import { ReactNode, FC } from "react";
import { OpenIDConnectConfig, IdToken } from '@epfl-si/appauth';
export interface ContextProps extends OpenIDConnectConfig {
    onNewToken?: (token: string) => void;
    onLogout?: () => void;
    onInitialAuthComplete?: () => void;
    minValiditySeconds?: number;
    children?: ReactNode;
}
export declare enum StateEnum {
    LoggedOut = 0,
    InProgress = 1,
    LoggedIn = 2,
    Error = 3
}
export interface State {
    state: StateEnum;
    error?: string;
    login: () => void;
    logout: () => void;
    accessToken?: string;
    idToken?: IdToken;
}
/**
 * OpenID-Connect state machine as a React context.
 *
 * Perform the operations dictated by the OpenID-Connect specification
 * once at the time the component is created, so as to determine
 * logged-in or logged-out status on behalf of {@link
 * useOpenIDConnectContext}. Said operations consist of the following
 * steps:
 *
 * - consume any OpenID-Connect credentials out
 *   of the browser URL bar (`code=`, `state=`, `error=` and
 *   `session_state=`), regardless of whether they are found in the
 *   “query” or “fragment” part
 * - redeem these OpenID-Connect credentials with the authorization
 *   server, in exchange for an authorization token and (optionally) an
 *   ID token
 * - communicate the success or failure of the previous steps through
 *   the `onInitialAuthComplete` callback and either the `onNewToken`
 *   or the `onLogout` callback
 * - arrange for timely token renewal
 *
 * This is a React context, meaning that one may use the matching
 * {@link useOpenIDConnectContext} React hook to consume the
 * authentication state from any descendant component.
 *
 * Note that contrary to a “typical” React component, an
 * `<OIDCContext>` does *not* destroy its state when its props change
 * (even “important” props like `authServerUrl`), as doing so would
 * typically result in logging the user out. If you do want that to
 * happen, you must arrange to call
 * `useOpenIDConnectContext().logout()` by yourself.
 */
export declare const OIDCContext: FC<ContextProps>;
/**
 * React hook to consume the state of the ancestor `<OIDCContext>`.
 *
 * Call this function from a React functional component that has an
 * `<OIDCContext>` as the ancestor (otherwise a dummy, invariable
 * state will be returned). The return value, `state`, is a `State`
 * object containing data indicating whether we are logged in
 * (`state.state`, `state.error`) and who is logged in
 * (`state.idToken`); as well as actuator methods that your UI (e.g.
 * your login button's `onClick`) might want to call (`state.login()`
 * and `state.logout()`).
 *
 * @function useOpenIDConnectContext
 */
export declare function useOpenIDConnectContext(): State;
