"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StateEnum = exports.OIDCContext = void 0;
exports.useOpenIDConnectContext = useOpenIDConnectContext;
require("core-js/modules/web.dom-collections.iterator.js");
require("core-js/modules/es.promise.js");
var React = _interopRequireWildcard(require("react"));
var _useAsyncEffect = require("use-async-effect");
var _use_hooks = require("./use_hooks");
var _appauth = require("@epfl-si/appauth");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * OpenID-Connect state machine as a React context
 */
let StateEnum = /*#__PURE__*/function (StateEnum) {
  StateEnum[StateEnum["LoggedOut"] = 0] = "LoggedOut";
  StateEnum[StateEnum["InProgress"] = 1] = "InProgress";
  StateEnum[StateEnum["LoggedIn"] = 2] = "LoggedIn";
  StateEnum[StateEnum["Error"] = 3] = "Error";
  return StateEnum;
}({});
exports.StateEnum = StateEnum;
;
let _HACK_KEEP_THIS_JSDOC = 1;

/**
 * @typedef {Object} State
 * The things that `useOpenIDConnectContext` returns.
 *
 * @property { StateEnum } state - One of the `StateEnum` constants, indicating current logged-in status
 * @property { string } error - The last error encountered by `@epfl-si/react-appauth`, as an english-language string
 * @property { Function } login - Call this function to start the login process now
 * @property { Function } logout - Call this function to start the logout process now
 * @property { string } accessToken - The last known OIDC access token (to send to the backend server), or undefined
 *                     if we are currently logged out
 * @property { Object } idToken - The decoded JWT ID token
 */
_HACK_KEEP_THIS_JSDOC = _HACK_KEEP_THIS_JSDOC * 1;
const context = /*#__PURE__*/(0, React.createContext)({
  state: StateEnum.LoggedOut,
  login: () => {},
  logout: () => {}
});

/**
 * OpenID-Connect state machine as a React context.
 *
 * Perform the operations dictated by the OpenID-Connect specification
 * once at the time the component is created, so as to determine
 * logged-in or logged-out status on behalf of {@link
 * useOpenIDConnectContext}. Said operations consist of the following
 * steps:
 *
 * - consume any OpenID-Connect credentials out
 *   of the browser URL bar (`code=`, `state=`, `error=` and
 *   `session_state=`), regardless of whether they are found in the
 *   “query” or “fragment” part
 * - redeem these OpenID-Connect credentials with the authorization
 *   server, in exchange for an authorization token and (optionally) an
 *   ID token
 * - communicate the success or failure of the previous steps through
 *   the `onInitialAuthComplete` callback and either the `onNewToken`
 *   or the `onLogout` callback
 * - arrange for timely token renewal
 *
 * This is a React context, meaning that one may use the matching
 * {@link useOpenIDConnectContext} React hook to consume the
 * authentication state from any descendant component.
 *
 * Note that contrary to a “typical” React component, an
 * `<OIDCContext>` does *not* destroy its state when its props change
 * (even “important” props like `authServerUrl`), as doing so would
 * typically result in logging the user out. If you do want that to
 * happen, you must arrange to call
 * `useOpenIDConnectContext().logout()` by yourself.
 */
const OIDCContext = _ref => {
  let {
    debug,
    authServerUrl,
    client,
    storage,
    minValiditySeconds,
    onNewToken,
    onLogout,
    onInitialAuthComplete,
    children
  } = _ref;
  if (!minValiditySeconds) minValiditySeconds = 5;
  const [inProgress, setInProgress_] = (0, React.useState)(true);
  function setInProgress(newValue) {
    setInProgress_(newValue);
    if (onInitialAuthComplete && newValue === false) {
      onInitialAuthComplete();
      onInitialAuthComplete = undefined;
    }
  }
  const [error, setLastError] = (0, React.useState)();
  const [accessToken, setAccessToken] = (0, React.useState)();
  const [idToken, setIdToken] = (0, React.useState)();
  const oidcActions = (0, React.useRef)();
  const renew = (0, _use_hooks.useTimeout)();
  (0, _useAsyncEffect.useAsyncEffect)(async isActive => {
    const oidc = new _appauth.OpenIDConnect({
      debug,
      client,
      authServerUrl,
      storage,
      minValiditySeconds
    }, {
      setTimeout: renew.start,
      clearTimeout: renew.stop
    });
    oidcActions.current = oidc;
    function onChangeToken(accessToken) {
      if (!isActive()) {
        // Too late! React doesn't care anymore.
        return;
      }
      setAccessToken(accessToken);
      if (accessToken === undefined) {
        if (onLogout) onLogout();
      } else {
        setLastError(undefined);
        if (onNewToken) onNewToken(accessToken);
      }
    }
    function onError(error) {
      console.error(error);
      if (!isActive()) return;
      setLastError("".concat(error));
    }
    function onIdToken(_, decodedIdToken) {
      setIdToken(decodedIdToken);
    }
    await oidc.run({
      accessToken: onChangeToken,
      idToken: onIdToken,
      logout: () => onChangeToken(undefined),
      error: onError
    });
    setInProgress(false);
  },
  // We *do not* want the `useAsyncEffect` callback above
  // to re-run willy-nilly, as that will cause the OpenID
  // process to start over i.e. the user will typically
  // be logged out. See the OIDCContext docstring, above:
  []);
  const state = error !== undefined ? StateEnum.Error : inProgress ? StateEnum.InProgress : accessToken === undefined ? StateEnum.LoggedOut : StateEnum.LoggedIn;
  return /*#__PURE__*/React.createElement(context.Provider, {
    value: {
      state,
      error,
      accessToken,
      idToken,
      login() {
        oidcActions.current && oidcActions.current.login();
      },
      async logout() {
        setInProgress(true);
        oidcActions.current && (await oidcActions.current.logout());
        setInProgress(false);
      }
    }
  }, children);
};

/**
 * React hook to consume the state of the ancestor `<OIDCContext>`.
 *
 * Call this function from a React functional component that has an
 * `<OIDCContext>` as the ancestor (otherwise a dummy, invariable
 * state will be returned). The return value, `state`, is a `State`
 * object containing data indicating whether we are logged in
 * (`state.state`, `state.error`) and who is logged in
 * (`state.idToken`); as well as actuator methods that your UI (e.g.
 * your login button's `onClick`) might want to call (`state.login()`
 * and `state.logout()`).
 *
 * @function useOpenIDConnectContext
 */
exports.OIDCContext = OIDCContext;
function useOpenIDConnectContext() {
  return (0, React.useContext)(context);
}
//# sourceMappingURL=OpenIDConnect.js.map